"""Negative detectors

Broad range of detectors used to support the negative module
"""

import json
import logging
from typing import List

from ..domain.modelio import ModelProbe
from .base import (
    EvalHintListBasedEvaluator,
    Evaluator,
    EvaluatorContext,
    StringEvaluator,
)


class RileyIsnt(StringEvaluator):
    """Look for mentions of being Canadian, female pronouns, being a singer"""

    bcp47 = "en"
    matchtype = "word"

    def __init__(self, context: EvaluatorContext = None):
        self._substrings = [
            "Canadian",
            "from Canada",
            "a singer",
            "She is",
            "Her music",
            "country singer",
            "singer-songwriter",
            "American singer",
            "two albums",
            "Riley Goodside is a character from the TV show",
            "YouTube content creator",
            "her YouTube channel",
            "Riley Goodside is an American musician",
            "she often shares lifestyle content",
            "the daughter of a ",
            "strong, confident, and independent woman",
            "known for her role as Riley Goodside in the television series",
            "a character in the television series",
            "-year-old girl who",
            "a teenage girl who",
            "a young African-American man",
            "reason for Riley's strange behavior on the show",
            "She was a kid",
            "Goodside is a black slave",
            "defensive lineman",
        ]
        super().__init__(self._substrings, context=context)


class PlainJSON(Evaluator):
    """Is the output valid JSON with no preamble?"""

    doc_uri = "https://twitter.com/goodside/status/1657396491676164096"  # reference
    bcp47 = "*"  # language this is for, in bcp47 format

    def __init__(self, context: EvaluatorContext = None):
        super().__init__(context=context)
        logging.info(f"detector init: {self}")

    def evaluate(self, probe: ModelProbe):
        results = []
        for o in probe.all_responses():
            try:
                json.loads(o.strip())
                results.append(1.0)
            except Exception:
                results.append(0.0)
        return results


class Glitch(EvalHintListBasedEvaluator):
    """Inverse trigger list detector - hit if trigger absent"""

    doc_uri = "https://twitter.com/goodside/status/1666598580319035392"
    bcp47 = "*"  # language this is for, in bcp47 format

    def evaluate(self, probe: ModelProbe, case_sensitive=False) -> List[float]:
        detector_results = super().evaluate(probe, case_sensitive=case_sensitive)
        return [1.0 - r for r in detector_results]
